-- scope stuff
qualityoflife = qualityoflife or {};
local q = qualityoflife;

QOL_OPTIONS = QOL_OPTIONS or {};

-- variables
local partyMembers, raidMembers;

-- init
function q:Init()
	if QOL_OPTIONS["OPT_SHOWCRF"] == nil then
		QOL_OPTIONS["OPT_SHOWCRF"] = false;
	end

	q.charName = UnitName("player");

	q.initFinished = true;
	q:Println(q:GetText("AddOn loaded. Type /qol for options."));
end

-- command handling
SLASH_QUALITYOFLIFE1 = "/qol";
local CMD_CRF = "crf";

q.commandHandlers = q.commandHandlers or {};
SlashCmdList["QUALITYOFLIFE"] = function(msg)
	local printLines = {};
	table.insert(printLines, "[Core] " .. q:GetText("Unknown command. Possible parameters are: /qol ..."));
	table.insert(printLines, "[Core] " .. q:GetText("... crf -> toggles whether default compact raid frames are displayed or not"));

	if msg then
		msg = q:Trim(msg);
		if msg == CMD_CRF then
			QOL_OPTIONS["OPT_SHOWCRF"] = not QOL_OPTIONS["OPT_SHOWCRF"];
			q:SetCrfVisible(QOL_OPTIONS["OPT_SHOWCRF"]);
			q:Println(q:GetText("Compact raid frames are displayed: ") .. q:BoolToYesNo(QOL_OPTIONS["OPT_SHOWCRF"]));
			return;
		end

		for _, v in ipairs(q.commandHandlers) do
			local result = v(msg);
			if not result then
				return;
			else
				for _, v in ipairs(result) do
					table.insert(printLines, v);
				end
			end
		end
	end
	
	for _, v in ipairs(printLines) do
		q:Println(v);
	end
end

-- event handling
q.eventFrame = q.eventFrame or CreateFrame("Frame");
q.eventFrame:RegisterEvent("ADDON_LOADED");
q.eventFrame:RegisterEvent("GROUP_ROSTER_UPDATE");
q.eventFrame:RegisterEvent("RAID_ROSTER_UPDATE");

--q.eventScripts = q.eventScripts or {};
--q.eventScripts["OnEvent"] = q.eventScripts["OnEvent"] or {};
q.eventFrame:SetScript("OnEvent", function(self, event, arg1)
	if event then
		if event == "ADDON_LOADED" and arg1 == "QoL_Core" and not q.initFinished then
			q.Init();
		elseif event == "GROUP_ROSTER_UPDATE" or event == "RAID_ROSTER_UPDATE" then
			q:ResetMemberInfo();
		elseif event == "PLAYER_ENTERING_WORLD" then
			q:SetCrfVisible(QOL_OPTIONS["OPT_SHOWCRF"]);
		end
	end
end);

-- core stuff
function q:SetCrfVisible(value)
	if _G["CompactRaidFrameContainer"] and _G["CompactRaidFrameContainer"]:IsVisible() ~= value then
		if value then
			_G["CompactRaidFrameContainer"]:Show();
		else
			_G["CompactRaidFrameContainer"]:Hide();
		end
	end
end

function q:GetPartyMembers()
	if not partyMembers then
		partyMembers = {};
		local plist = GetHomePartyInfo();

		if plist then
			local n = 1;
			for _, v in pairs(plist) do
				partyMembers[n] = v;
				n = n + 1;
			end
		end
	end
	return partyMembers;
end

function q:GetRaidMembers()
	if not raidMembers then
		raidMembers = {};
		raidMembers[0] = {};
		local noRaidMembers = GetNumGroupMembers();

		if noRaidMembers then
			for n = 1, noRaidMembers do
				local name, rank, subgroup, level, class, fileName, zone, online, isDead, role, isML = GetRaidRosterInfo(n);

				-- table of all raid members by id
				raidMembers[0][n] = name;

				-- table of all raid members, mapped by subgroups
				if not raidMembers[subgroup] then
					raidMembers[subgroup] = {};
				end
				local subGroupId = table.getn(raidMembers[subgroup]) + 1;
				raidMembers[subgroup][subGroupId] = name;

				n = n + 1;
			end
		end
	end
	return raidMembers;
end

-- misc stuff
function q:Println(str, module)
	if str then
		if module then
			print("[QoL] [" .. module .. "] " .. str);
		else
			print("[QoL] " .. str);
		end
	end
end

function q:StartsWith(str, substr, caseInsensitive)
	if caseInsensitive then
		str = string.upper(str);
		substr = string.upper(substr);
	end

   return string.sub(str, 1, string.len(substr)) == substr;
end

function q:StringContains(str, match, caseInsensitive)
	if caseInsensitive then
		str = string.upper(str);
		match = string.upper(match);
	end

	return string.find(str, "%" .. match);
end

function q:Trim(str)
	if str then
		return str:gsub("^%s*(.-)%s*$", "%1");
	end
	return "";
end

function q:ResetMemberInfo()
	partyMembers = nil;
	raidMembers = nil;
end

function q:BoolToYesNo(value)
	if value then
		return q:GetText("yes");
	end
	return q:GetText("no");
end

function q:GetTableSize(t)
    local count = 0
    for _, __ in pairs(t) do
        count = count + 1
    end
    return count
end

function q:GetSortedTableKeys(t, descending)
	if t then
		local keys = {};
		for k, _ in pairs(t) do
			local num = tonumber(k);
			if num then
				table.insert(keys, num);
			else
				table.insert(keys, k);
			end
		end
		if descending then
			table.sort(keys, function(a, b) return a > b end); -- descending
		else
			table.sort(keys);
		end
		return keys;
	end
end